#ifndef _CommonDialog_CPP
#define _CommonDialog_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

#include "../Resources/Resource.H"

#include "../Headers/NSWFL.H"
#include "../Headers/Package.H"
#include "../Headers/Entry.H"
#include "../Headers/CommonDialog.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char **AlreadyLoaded;
char gsExtractLocation[255 + 1];

HIMAGELIST hImageList = NULL;

int LoadedItems = 0;
int MaxLoadedItems = 0;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

HTREEITEM TVAddItem(HWND treehwnd, HTREEITEM iparent, char *itext, int Image, int SelImage)
{
    TV_INSERTSTRUCT tvis;

    tvis.item.hItem          = iparent;
    tvis.item.pszText        = itext;
    tvis.item.iImage         = Image;
    tvis.item.iSelectedImage = SelImage;
    tvis.item.mask           = TVIF_TEXT|TVIF_SELECTEDIMAGE|TVIF_IMAGE;
    tvis.hParent             = iparent;
    tvis.item.pszText        = itext;
    tvis.item.cchTextMax     = strlen(itext);

    return TreeView_InsertItem(treehwnd, &tvis);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool GetDesktopFolderPath(char *Desktop)
{
    DWORD SizeOfBuffer = MAX_PATH;

    if(IsWinNT())
    {
        if(Get_StringRegistryValue(HKEY_CURRENT_USER, SHELLFOLDERS, "Desktop", Desktop, SizeOfBuffer))
        {
            return true;
        }
        else return false;
    }
    else{
        if(Get_StringRegistryValue(HKEY_LOCAL_MACHINE, SHELLFOLDERS, "Common Desktop", Desktop, SizeOfBuffer))
        {
            return true;
        }
        else return false;
    }

    return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool IsAlreadyLoaded(char *inFolder)
{
    int LocalLoop = 0;

    while(LocalLoop < LoadedItems)
    {
        if(strcmpi(inFolder, AlreadyLoaded[LocalLoop]) == 0)
        {
            return true;
        }
        LocalLoop++;
    }
    return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool InitAlreadyLoaded(void)
{
    int LocalLoop = 0;

    AlreadyLoaded = (char **) calloc(sizeof(char *), LOAD_TREE_ITEMS_INCREMENT);

    MaxLoadedItems = LOAD_TREE_ITEMS_INCREMENT;

    LoadedItems = 0;

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FreeAlreadyLoaded(void)
{
    int LocalLoop = 0;

    while(LocalLoop < LoadedItems)
    {
        free(AlreadyLoaded[LocalLoop]);
        AlreadyLoaded[LocalLoop] = NULL;
        LocalLoop++;
    }

    LoadedItems = 0;
    MaxLoadedItems = 0;

    free(AlreadyLoaded);

    AlreadyLoaded = NULL;

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool IndexDirectoryForSubItem(HWND hTreeHandle, HTREEITEM SubLeaf, char *inFolder, bool IndexTwoDeep)
{
    if(IsAlreadyLoaded(inFolder))
    {
        return true;
    }

    WIN32_FIND_DATA FindData;
    HANDLE FileHandle = NULL;

    char DirTemp[MAX_PATH + 1];

    if(inFolder[strlen(inFolder) - 1] != '\\')
        sprintf(DirTemp, "%s\\*.*", inFolder);
    else sprintf(DirTemp, "%s*.*", inFolder);

    if((FileHandle = FindFirstFile(DirTemp, &FindData)) == INVALID_HANDLE_VALUE)
    {
        return false;
    }

    if(LoadedItems == MaxLoadedItems)
    {
        AlreadyLoaded = (char **) realloc(AlreadyLoaded, (MaxLoadedItems + LOAD_TREE_ITEMS_INCREMENT) * sizeof(char *));
        MaxLoadedItems = (MaxLoadedItems + LOAD_TREE_ITEMS_INCREMENT);
    }

    AlreadyLoaded[LoadedItems] = (char *) calloc(sizeof(char), strlen(inFolder) + 1);
    strcpy(AlreadyLoaded[LoadedItems], inFolder);
    LoadedItems++;

    do{
        if( strcmp(FindData.cFileName, ".") && strcmp(FindData.cFileName, "..") )
        {
            if(FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
            {
                HTREEITEM RootLeaf = TVAddItem(hTreeHandle, SubLeaf, FindData.cFileName, 8, 9);

                if(IndexTwoDeep)
                {
                    if(inFolder[strlen(inFolder) - 1] != '\\')
                        sprintf(DirTemp, "%s\\%s", inFolder, FindData.cFileName);
                    else sprintf(DirTemp, "%s%s", inFolder, FindData.cFileName);

                    IndexDirectoryForSubItem(hTreeHandle, RootLeaf, DirTemp, false);
                }
            }
        }
    } while(FindNextFile(FileHandle, &FindData));

    FindClose(FileHandle);

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool PopBrowseTree(HWND hTreeHandle)
{
    HTREEITEM DesktopLeaf, MyComputerLeaf, SubLeaf;

    int DriveLoop = 65;  //Drive A
    char PathTemp[MAX_PATH + 1];
    char DriveText[5];
    int DriveType = 0;
    int Image = 0;

    DesktopLeaf = TVAddItem(hTreeHandle, TVI_ROOT, "Desktop", 6, 6);
    MyComputerLeaf = TVAddItem(hTreeHandle, DesktopLeaf, "MyComputer", 10, 10);

    while(DriveLoop < 90) //Drive Z
    {
        sprintf(DriveText, "%c:", DriveLoop);
        if((DriveType = GetDriveType(DriveText)) != 1)
        {
            if(DriveType == 0) Image = 0; //Unknown Type
            if(DriveType == 2) Image = 1; //Removable Disk
            if(DriveType == 3) Image = 2; //Fixed Disk
            if(DriveType == 4) Image = 3; //Remote Network Drive
            if(DriveType == 5) Image = 4; //Cd-Rom Drive
            if(DriveType == 6) Image = 5; //High Speed RAM Drive

            SubLeaf = TVAddItem(hTreeHandle, MyComputerLeaf, DriveText, Image, Image);

            if(DriveType == 3)
            {
                IndexDirectoryForSubItem(hTreeHandle, SubLeaf, DriveText, false);
            }
        }

        DriveLoop++;
    }

    if(GetDesktopFolderPath(PathTemp))
    {
        IndexDirectoryForSubItem(hTreeHandle, DesktopLeaf, PathTemp, true);
    }

    TreeView_Expand(hTreeHandle, DesktopLeaf, TVE_EXPAND);
    TreeView_Expand(hTreeHandle, MyComputerLeaf, TVE_EXPAND);

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CreateTreeImageList(HWND hTreeHandle)
{
    hImageList = ImageList_Create(16, 16, ILC_COLOR24, 10, 1);

    HBITMAP hBitMap = NULL;

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_UNKNOWN));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_REMOVABLE));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_FIXED));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_NETWORK));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CDROM));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_RAMDRIVE));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_DESKTOP));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_MYDOCS));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CLOSEDFOLDER));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_OPENFOLDER));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_MYCOMPUTER));
    ImageList_Add(hImageList, hBitMap, hBitMap);
    DeleteObject(hBitMap);

    TreeView_SetImageList(hTreeHandle, hImageList, TVSIL_NORMAL);

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif

